# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class DictTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_empty_dict1(self):
        v = loads('{}')[0]
        self.assertEqual(type(v), dict)
        self.assertEqual(v, {})
        s = dumps([v])
        self.assertEqual(s, '{}')
    #
    def test_empty_dict2(self):
        v = loads('{}\n')[0]
        self.assertEqual(type(v), dict)
        self.assertEqual(v, {})
        s = dumps([v], pretty=True)
        self.assertEqual(s, '{}')
    #
    def test_dict1(self):
        v = loads('{a:1 b:2}')[0]
        self.assertEqual(type(v), dict)
        self.assertEqual(list(sorted(v.items())), [('a',1),('b',2)])
    #
    def test_dict1_1(self):
        v = loads('{a:1, b:2}', json=True)[0]
        self.assertEqual(type(v), dict)
        self.assertEqual(list(sorted(v.items())), [('a',1),('b',2)])
    #
    def test_dict2(self):
        v = loads('{a:[1,2] b:{c:3}}', json=True)[0]
        self.assertEqual(type(v), dict)
        self.assertEqual(list(sorted(v.items())), [('a',[1,2]),('b',{'c':3})])

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DictTestCase))
    return suite



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from axon._loader import as_name, factory
from axon._objects import undef #, NamedValue, named
from axon._objects import as_unicode, as_list, as_dict, as_tuple
from axon._dumper import reduce, dump_as_str





# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps
try:
    from cdecimal import Decimal as decimal
except:
    from decimal import Decimal as decimal

class DecimalTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_decimal1(self):
        v = loads('0$')[0]
        self.assertAlmostEqual(v, decimal('0.'))
    def test_decimal2(self):
        v = loads('0.$')[0]
        self.assertAlmostEqual(v, decimal('0.'))
    def test_decimal3(self):
        v = loads('123$')[0]
        self.assertAlmostEqual(v, decimal('123'))
    def test_decimal4(self):
        v = loads('3.141528$')[0]
        self.assertAlmostEqual(v, decimal('3.141528'))
    def test_decimal5(self):
        v = loads('-1.2345e5$')[0]
        self.assertAlmostEqual(v, decimal('-1.2345e5'))
    def test_decimal6(self):
        v = loads('-1.2345e+5$')[0]
        self.assertAlmostEqual(v, decimal('-1.2345e5'))
    def test_decimal7(self):
        v = loads('-1.2345e-5$')[0]
        self.assertAlmostEqual(v, decimal('-1.2345e-5'))
    def test_decimal8(self):
        v = loads('-1.2345e27$')[0]
        self.assertAlmostEqual(v, decimal('-1.2345e27'))
    def test_decimal9(self):
        v = loads('-1.2345e-27$')[0]
        self.assertAlmostEqual(v, decimal('-1.2345e-27'))
    def test_decimal10(self):
        v = loads('1.2345e+27$')[0]
        self.assertAlmostEqual(v, decimal('1.2345e+27'))
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DecimalTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps

try:
    import builtins
except ImportError:
    import __builtin__ as builtins

try:
    _builtins_unicode = builtins.unicode
    _builtins_str = builtins.str
    _chr = unichr
except AttributeError:
    _builtins_unicode = builtins.str
    _builtins_str = builtins.str
    _chr = chr

class Base64TestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_1(self):
        from random import randint
        for i in range(10):
            text = ''.join([_chr(randint(1,255)) for i in range(256)])
            if type(text) is _builtins_unicode:
                btext = text.encode('latin1')
            else:
                btext = bytes(text)
            text1= dumps([btext])
            print('***', btext, '***')
            print('***', text1, '***')
            btext1 = loads(text1)[0]
            self.assertEqual(btext, btext1)
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Base64TestCase))
    return suite




# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class MappingTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_empty_mapping(self):
        v = mapping('aaa', {})
        self.assertEqual(v.name, 'aaa')
        self.assertFalse(v.mapping)
        self.assertEqual(v.mapping, {})
        self.assertFalse(v.sequence)
        self.assertEqual(v.sequence, None)
    #
    def test_mapping_init(self):
        l = {'a':1, 'b':2}
        v = mapping('aaa', l)
        self.assertEqual(v.mapping, {'a':1, 'b':2})
    #
    def test_mapping1(self):
        v = loads('aaa{a:1 b: 2 c : 3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Mapping)
        self.assertEqual(v.sequence, None)
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v])
        self.assertEqual(s, '''aaa{a:1 b:2 c:3}''')
    #
    def test_mapping1_1(self):
        v = loads('''
aaa:
   a:1
   b: 2
   c : 3
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Mapping)
        self.assertEqual(v.sequence, None)
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v])
        self.assertEqual(s, '''aaa{a:1 b:2 c:3}''')
    #
    def test_mapping2(self):
        v = loads('aaa {a: 1 b:2 c:3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Mapping)
        self.assertEqual(v.sequence, None)
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  a: 1
  b: 2
  c: 3''')
    #
    def test_mapping3(self):
        v = loads('''
aaa:
  a: 1
  b: 2
  c: c:
    e: 1
    f: f:
      r: 2
      s: 3
  d: 4
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Mapping)
        self.assertEqual(v.sequence, None)
        s = dumps([v])
        self.assertEqual(s, '''aaa{a:1 b:2 c:c{e:1 f:f{r:2 s:3}} d:4}''')
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  a: 1
  b: 2
  c: c:
    e: 1
    f: f:
      r: 2
      s: 3
  d: 4''')
    #
    def test_mapping4(self):
        v = loads('''
aaa:
  a: 1
  b: 2
  c:
    e: 1
    f:
      r: 2
      s: 3
  d:
    4
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Element)
        s = dumps([v])
        self.assertEqual(s, '''aaa{a:1 b:2 c{e:1 f{r:2 s:3}} d{4}}''')
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  a: 1
  b: 2
  c:
    e: 1
    f:
      r: 2
      s: 3
  d:
    4''')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MappingTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class SequenceTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_empty_sequence(self):
        v = sequence('aaa', [])
        self.assertEqual(v.name, 'aaa')
        self.assertFalse(v.sequence)
        self.assertEqual(v.sequence, [])
        self.assertFalse(v.mapping)
        self.assertEqual(v.mapping, None)
    #
    def test_sequence_init2(self):
        l = [1,2,3]
        v = sequence('aaa', l)
        self.assertEqual(v.sequence, l)
    #
    def test_sequence1(self):
        v = loads('aaa{1 2 3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        self.assertEqual(v.sequence, [1, 2, 3])
        self.assertEqual(v.mapping, None)
        s = dumps([v])
        self.assertEqual(s, 'aaa{1 2 3}')
    #
    def test_sequence1_1(self):
        v = loads('''
aaa:
  1
  2
  3
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        self.assertEqual(v.sequence, [1, 2, 3])
        self.assertEqual(v.mapping, None)
        s = dumps([v])
        self.assertEqual(s, 'aaa{1 2 3}')
    #
    def test_sequence1_2(self):
        v = loads('''
aaa:
    1 2
    3 4
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        self.assertEqual(v.sequence, [1, 2, 3, 4])
        self.assertEqual(v.mapping, None)
        s = dumps([v])
        self.assertEqual(s, 'aaa{1 2 3 4}')
    #
    def test_sequence1_3(self):
        v = loads('''
aaa:
  1 2
  bbb {1 2 3}
  3 4
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        #self.assertEqual(v.sequence, [1, 2, 3, 4])
        self.assertEqual(v.mapping, None)
        s = dumps([v])
        self.assertEqual(s, 'aaa{1 2 bbb{1 2 3} 3 4}')
    #
    def test_sequence1_4(self):
        v = loads('''
aaa:
  1 2
  bbb:
    1 2
    3
  3 4
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        #self.assertEqual(v.sequence, [1, 2, 3, 4])
        self.assertEqual(v.mapping, None)
        s = dumps([v], pretty=False)
        self.assertEqual(s, 'aaa{1 2 bbb{1 2 3} 3 4}')
    #
    def test_sequence2(self):
        v = loads('aaa{1 2 3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Sequence)
        self.assertEqual(v.sequence, [1, 2, 3])
        self.assertEqual(v.mapping, None)
        s = dumps([v], pretty=2)
        self.assertEqual(s, \
'''aaa:
  1
  2
  3''')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SequenceTestCase))
    return suite



# -*- coding: utf-8 -*-
from __future__ import print_function

import json
import xon
import random
import time
import unittest
from xon.types import unicode_type
import gc

def random_string(n):
    text = ''.join(chr(ord('a')+random.randint(1,20)) for i in range(20))
    text = xon.as_unicode(text)
    #assert type(text) is unicode_type
    return text

class SimpleONvsJSONTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_vs_json_1(self):
        lst = []
        for i in range(4000):
            lst.append([
                random_string(8), random.random(),
                random_string(8), random.randint(1,99999),
                random_string(8), random_string(32),
                random_string(8), random.random(),
                random_string(8), random.randint(1,99999),
                random_string(8), random_string(32),
                random_string(8), random.random(),
                random_string(8), random.randint(1,99999),
                random_string(8), random_string(32),
                random_string(8), random.random(),
                random_string(8), random.randint(1,99999),
                random_string(8), random_string(32),
            ])

        xon_text = xon.dumps(lst)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_2(self):
        lst = []
        for i in range(4000):
            lst.append({
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
            })

        xon_text = xon.dumps(lst)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_3(self):
        d = {}
        for j in range(70):
            lst = []
            for i in range(100):
                lst.append({
                    random_string(8): random.random(),
                    random_string(8): random.randint(1,99999),
                    random_string(8): random_string(32),
                    random_string(8): random.random(),
                    random_string(8): random.randint(1,99999),
                    random_string(8): random_string(32),
                    random_string(8): random.random(),
                    random_string(8): random.randint(1,99999),
                    random_string(8): random_string(32),
                })
            d[random_string(8)] = lst

        xon_text = xon.dumps([d])
        #print(xon_text)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(d)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_4(self):
        lst = []
        for i in range(4000):
            lst.append([
                random_string(8), random_string(32),
                random_string(32), random_string(8),
                random_string(8), random_string(32),
                random_string(32), random_string(32),
                random_string(8), random_string(8),
                random_string(32), random_string(32),
                random_string(8), random_string(8),
                random_string(32), random_string(8),
                random_string(8), random_string(32),
                random_string(32), random_string(8),
                random_string(8), random_string(8),
                random_string(32), random_string(32),
                random_string(8), random_string(8),
                random_string(32), random_string(8),
                random_string(8), random_string(32),
            ])

        xon_text = xon.dumps(lst)
        #print(xon_text)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_5(self):
        lst = []
        for i in range(5000):
            lst.extend([
                random_string(8), random_string(8),
                random_string(32), random_string(8),
                random_string(32), random_string(32),
                random_string(8), random_string(8),
                random_string(8), random_string(8),
                random_string(32), random_string(8),
                random_string(8), random_string(32),
                random_string(32), random_string(8),
                random_string(32), random_string(32),
                random_string(8), random_string(8),
                random_string(32), random_string(8),
                random_string(32), random_string(32),
            ])

        xon_text = xon.dumps(lst)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_6(self):
        lst = []
        for i in range(5000):
            lst.extend([
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
                random.randint(1,99999), random.randint(1,99999),
            ])
        lst = [lst]

        xon_text = xon.dumps(lst)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

    def test_vs_json_7(self):
        lst = []
        for i in range(5000):
            lst.extend([
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
                random.random(), random.random(),
            ])
        lst = [lst]

        xon_text = xon.dumps(lst)
        ldr = xon.iloads(xon_text)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = ldr.load()
        dt_xon = time.time() - t0
        gc.enable()

        json_text = json.dumps(lst)
        gc.collect()
        gc.disable()
        t0 = time.time()
        v = json.loads(json_text)
        dt_json = time.time() - t0
        gc.enable()

        print('\nxon:', dt_xon, 'json', dt_json)

        #self.assertTrue(dt_xon < dt_json)

def run():
    inst = SimpleONvsJSONTestCase()
    inst.test_vs_json_1()
    inst.test_vs_json_2()
    inst.test_vs_json_3()
    inst.test_vs_json_4()
    inst.test_vs_json_5()

#run()



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps

class FloatTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_float1(self):
        v = loads('0.')[0]
        self.assertAlmostEqual(v, 0.)
    def test_float2(self):
        v = loads('3.141528')[0]
        self.assertAlmostEqual(v, 3.141528)
    def test_float3(self):
        v = loads('-1.2345e5')[0]
        self.assertAlmostEqual(v, -1.2345e5)
    def test_float4(self):
        v = loads('-1.2345e+5')[0]
        self.assertAlmostEqual(v, -1.2345e5)
    def test_float5(self):
        v = loads('-1.2345e-5')[0]
        self.assertAlmostEqual(v, -1.2345e-5)
    def test_float6(self):
        v = loads('-1.2345e27')[0]
        self.assertAlmostEqual(v, -1.2345e27)
    def test_float7(self):
        v = loads('-1.2345e-27')[0]
        self.assertAlmostEqual(v, -1.2345e-27)
    def test_zero(self):
        v = loads('-0.')[0]
        s = dumps([v])
        self.assertEqual(s, '-0.0')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FloatTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps

class IntTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_int1(self):
        v = loads('0')[0]
        self.assertEqual(v, 0)
        s = dumps([v])
        self.assertEqual(s, '0')
    def test_int1a(self):
        v = loads('-0')[0]
        self.assertEqual(v, 0)
        s = dumps([v])
        self.assertEqual(s, '0')
    def test_int2(self):
        v = loads('-1')[0]
        self.assertEqual(v, -1)
        s = dumps([v])
        self.assertEqual(s, '-1')
    def test_int3(self):
        v = loads('1')[0]
        self.assertEqual(v, 1)
        s = dumps([v])
        self.assertEqual(s, '1')
    def test_int4(self):
        v = loads('17171717171717171717171717')[0]
        self.assertEqual(v, 17171717171717171717171717)
        s = dumps([v])
        self.assertEqual(s, '17171717171717171717171717')
    def test_int5(self):
        v = loads('17')[0]
        self.assertEqual(v, 17)
        s = dumps([v])
        self.assertEqual(s, '17')

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(IntTestCase))
    return suite



# coding: utf-8

import unittest

from axon.test.test_int import *
from axon.test.test_float import *
from axon.test.test_decimal import *
from axon.test.test_string import *
from axon.test.test_constant import *
from axon.test.test_datetime import *
from axon.test.test_list import *
from axon.test.test_dict import *
from axon.test.test_tuple import *
#from axon.test.test_set import *
from axon.test.test_element import *
from axon.test.test_mapping import *
from axon.test.test_sequence import *
from axon.test.test_instance import *
from axon.test.test_safe_loads import *
from axon.test.test_unsafe_loads import *
from axon.test.test_crossref import *
from axon.test.test_base64 import *
from axon.test.test_errors import *

unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
from __future__ import print_function

import yaml
import random
import time
import unittest
import xon

def random_string(n):
    text = ''.join([chr(ord('a')+random.randint(1,20)) for i in range(20)])
    return xon.as_unicode(text)

class SimpleONvsYAMLTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_vs_yaml_1(self):
        lst = []
        for i in range(1000):
            lst.append({
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
                random_string(8): random.random(),
                random_string(8): random.randint(1,99999),
                random_string(8): random_string(32),
            })
        yaml_text = yaml.dump(lst, Dumper=yaml.CDumper)
        t0 = time.time()
        v = yaml.load(yaml_text, Loader=yaml.CLoader)
        dt_yaml = time.time() - t0

        xon_text = xon.dumps([lst], quote=True, pretty=False)
        t0 = time.time()
        v = xon.loads(xon_text)
        dt_xon = time.time() - t0

        print('\nxon:', dt_xon, 'yaml', dt_yaml)

        self.assertTrue(dt_xon < dt_yaml)

    def test_vs_yaml_2(self):
        d = {}
        for j in range(50):
            lst = []
            for i in range(100):
                lst.append({
                    random_string(8): random.random(),
                    random_string(8): random.randint(1,99999),
                    random_string(8): random_string(32),
                    random_string(8): random.random(),
                    random_string(8): random.randint(1,99999),
                    random_string(8): random_string(32),
                })
            d[random_string(8)] = lst
        yaml_text = yaml.dump(d, Dumper=yaml.CDumper)
        t0 = time.time()
        v = yaml.load(yaml_text, Loader=yaml.CLoader)
        dt_yaml = time.time() - t0

        xon_text = xon.dumps([d], quote=True, pretty=False)
        #print(xon_text)
        t0 = time.time()
        v = xon.loads(xon_text)
        dt_xon = time.time() - t0

        print('\nxon:', dt_xon, 'yaml', dt_yaml)

        self.assertTrue(dt_xon < dt_yaml)

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SimpleONvsYAMLTestCase))
    return suite







# {{LICENCE}}

from __future__ import unicode_literals
from axon._loader import Loader
from axon._dumper import Dumper
from axon.types import unicode_type, str_type
from axon._objects import as_unicode, StringReader

try:
    from io import StringIO
except ImportError:
    from cStringIO import StringIO

class Reader(object):
    def __init__(self, fd, encoding):
        self.fd = fd
        self.encoding = encoding

    def readline(self):
        line = fd.readline()
        return line.decode(encoding)

    def close(self):
        self.fd.close()

def display(text, pretty=1, nsize=3, sorted=1):
    '''
    Display SimpleON text in formatted form for easy read.

    :param text:
       `unicode` SimpleON text

    :returns:
        formatted `unicode` SimpleON text
    '''
    val = loads(text)
    print(dumps(val, pretty=pretty, crossref=True, nsize=nsize, sorted=1))

def dumps(val, pretty=0, crossref=False, quote=False, nsize=0, sorted=1):
    '''
    Dump value into unicode text.

    :param val:
       value to convert

    :param simple_dumpers:
       dictionary containing callables for text representation of values of simple types &em;
       int, float, decimal, boolean, unicode text, date/time/datetime &em; in form of
       `type`:`dumper`.

    :param type_reducers:
       dictionary containing callables, which returns reduced data for
       representation in SimpleON.

    :param pretty:
        flag indicating pretty dumping:

        * True - use pretty dumping
        * False - do not use pretty dumping (default)

    :param crossref:
        flag for crossreferece support in unsafe mode

        * True - use crosserefernce support
        * False - do not use crosserefernce support (default)

    :returns:
        unicode string of SimpleON representation of `val`.
    '''
    try:
        from io import StringIO
    except ImportError:
        from cStringIO import StringIO

    fd = StringIO('')

    dumper = Dumper(fd, crossref, quote, pretty, nsize, sorted)
    dumper.dump(val)
    v = fd.getvalue()
    fd.close()
    return v

def dump(fpath, val, encoding='utf-8', pretty=0, crossref=False, quote=False, nsize=0, sorted=1):
    '''\
    Same as :py:func:`dumps` but for dumping into a file.

    :param fpath:
        path to file for dumping.

    For other parameters see :py:func:`dumps`.
    '''
    fd = open(fpath, mode='wt', encoding=encoding)
    dumper = Dumper(fd, crossref, quote, pretty, nsize, sorted)
    dumper.dump(val, pretty)
    fd.close()

def loads(text, mode="safe", errto=None, json=False):
    '''\
    Load values from unicode text.

    :param text:
        `unicode` text in SimpleON.

    :param mode:
        specifies the method of building python objects for complex values
        (see .. py:func:`load`)

    :param errto:
        name of file for reporting errors

    :param json:
        if true then allow to load from json format

    :returns:
        `list` of values.
    '''
    loader = iloads(text, mode, errto, json)
    return loader.load()

def iloads(text, mode="safe", errto=None, json=False):
    '''\
    Iterative loading values from unicode text.

    :param text:
        unicode text in SimpleON.

    :param mode:
        specifies the method of building python objects for complex values
        (see .. py:func:`load`)

    :param errto:
        name of file for reporting errors

    :param json:
        if true then allow to load from json format

    :returns:
        iterator object.
    '''
    text = as_unicode(text)
    #fd = StringIO(text)
    fd = StringReader(text)

    return iload(fd, mode, errto, json=json)

def load(fd, mode="safe", errto=None, encoding='utf-8', json=False):
    '''
    Load object from unicode text in SimpleON.

    :param fd:
        input file name of file object opening as TextIO/StringIO

    :param mode:
        specifies the method of building python objects for complex values.

    :param errto:
        name of file for reporting errors

    :param encoding:
        encoding of input file (default `utf-8`)

    There are values of parameter `mode`:
        * "safe" - use safe object builder
        * "strict" - use unsafe builder with strict name resolution: if there is no registered name
          for building object builder return `undef` object
        * "mixed" - use unsafe object builder with following rule: if there is no registered name
          for building object then use `safe` mode.

    :param json:
        if true then allow to load from json format

    :returns:
        `list` of values.
    '''
    loader = iload(fd, mode, errto, encoding, json)
    return loader.load()

def iload(fd, mode="safe", errto=None, encoding='utf-8', json=False):
    '''\
    Iterative loading values from input file.

    Arguments are same as in .. py:func:`load`.

    :returns:
        iterator object.
    '''

    if type(fd) in (str_type, unicode_type):
        try:
            from io import open
            fd = open(fd, mode='rt', encoding=encoding)
        except ImportError:
            fd = open(fd, mode='rt')
            fd = Reader(fd, encoding)

    loader = Loader(fd, mode, errto, json)

    return loader




# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class ObjectTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_empty_object(self):
        v = instance('aaa', [], {})
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(v.mapping, {})
        self.assertEqual(v.sequence, ())
    #
    def test_object_init(self):
        l = {'a':1, 'b':2}
        v = instance('aaa', [1,2,3], {'a':1, 'b':2})
        self.assertEqual(v.mapping, {'a':1, 'b':2})
        self.assertEqual(v.sequence, (1,2,3))
    #
    def test_object1(self):
        v = loads('aaa {4 5 6 a:1 b:2 c:3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Instance)
        self.assertEqual(v.sequence, (4, 5, 6))
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v])
        self.assertEqual(s, '''aaa{4 5 6 a:1 b:2 c:3}''')
    #
    def test_object2(self):
        v = loads('aaa{4 5 6 a:1 b : 2 c:3}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Instance)
        self.assertEqual(v.sequence, (4, 5, 6))
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  4
  5
  6
  a: 1
  b: 2
  c: 3''')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ObjectTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

pretty = False

class TupleTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_empty_tuple1(self):
        v = loads('()')[0]
        self.assertEqual(type(v), tuple)
        self.assertEqual(v, ())
        s = dumps([v])
        self.assertEqual(s, '()')
    #
    def test_empty_tuple2(self):
        v = loads('()')[0]
        self.assertEqual(type(v), tuple)
        self.assertEqual(v, ())
        s = dumps([v], pretty=True)
        self.assertEqual(s, '()')
    #
    def test_tuple1(self):
        v = loads('(1 2)')[0]
        self.assertEqual(type(v), tuple)
        self.assertEqual(v, (1,2))
    #
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TupleTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps

class StringTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_str1(self):
        v = loads('""')[0]
        self.assertEqual(v, '')
        s = dumps([v])
        self.assertEqual(s, '""')
    def test_str2(self):
        v = loads('"\n"')[0]
        self.assertEqual(v, '\n')
        s = dumps([v])
        self.assertEqual(s, '"\n"')
    def test_str3(self):
        t = '''"\
abc\
def"'''
        v = loads(t)[0]
        self.assertEqual(v, 'abcdef')
        s = dumps([v])
        self.assertEqual(s, '"abcdef"')
    def test_str4(self):
        v = loads('"abc абв"')[0]
        self.assertEqual(v, 'abc абв')
        s = dumps([v])
        self.assertEqual(s, '"abc абв"')
    def test_str5(self):
        t = '"ab\tcd\nef"'
        v = loads(t)[0]
        self.assertEqual(v, 'ab\tcd\nef')
        s = dumps([v])
        self.assertEqual(s, t)
    def test_str6(self):
        t = '''"abfdsfdfd
sdfdfdfdfsdfdwrerwrwerwe
cvbnbnfrtt"'''
        v = loads(t)[0]
        self.assertEqual(v, '''abfdsfdfd
sdfdfdfdfsdfdwrerwrwerwe
cvbnbnfrtt''')
        s = dumps([v])
        self.assertEqual(s, t)
    def test_str(self):
        t = '"\\u0041\\U0042"'
        v = loads(t)[0]
        self.assertEqual(v, "AB")
        s = dumps([v])
        self.assertEqual(s, '"AB"')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(StringTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class C(object):
    pass

class D(object):
    pass

class E(object):
    def __init__(self, a, b, c):
        self.a = a
        self.b = b
        self.c = c

def mapping_instance_maker(cls):
    def make_instance(attrs):
        inst = cls.__new__(cls)
        for name, value in attrs.items():
            setattr(inst, name, value)
        return inst
    return make_instance

def mapping_reducer_maker(cls):
    def type_reducer(o):
        attrs = {}
        for name in o.__dict__:
            if name.startswith('_'):
                continue
            attrs[as_name(name)] = getattr(o, name)

        return as_name(cls.__name__), attrs
    return type_reducer


factory('C', mapping_instance_maker(C))
factory('D', mapping_instance_maker(D))
factory('E', mapping_instance_maker(E))

reduce(C, mapping_reducer_maker(C))
reduce(D, mapping_reducer_maker(D))
reduce(E, mapping_reducer_maker(E))

class UnsafeLoadsTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_1(self):
        v = C()
        v.a = 1
        v.b = 2
        v.c = 3
        text = dumps([v])
        #display(text)
        v1 = loads(text, mode='strict')[0]
        self.assertEqual(v.a, v1.a)
        self.assertEqual(v.b, v1.b)
        self.assertEqual(v.c, v1.c)
    #
    def test_2(self):
        v = C()
        v.a = 1
        v.b = 2
        v.c = 3
        w = D()
        w.a = 'a'
        w.b = [1,2]
        w.c = 2
        text = dumps([v, w])
        #display(text)
        v1, w1 = loads(text, mode='strict')
        self.assertEqual(v.a, v1.a)
        self.assertEqual(v.b, v1.b)
        self.assertEqual(v.c, v1.c)
        self.assertEqual(w.a, w1.a)
        self.assertEqual(w.b, w1.b)
        self.assertEqual(w.c, w1.c)
    #
    def test_3(self):
        v = E(1, 2, 3)
        text = dumps([v])
        #display(text)
        v1 = loads(text, mode='strict')[0]
        self.assertEqual(v.a, v1.a)
        self.assertEqual(v.b, v1.b)
        self.assertEqual(v.c, v1.c)
    #
    def test_4(self):
        from random import randint
        v = C()
        v.x = 1
        v.y = lst = []
        for i in range(7):
            w = D()
            w.z = randint(1,1000)
            lst.append(w)
        text = dumps([v])
        #print()
        #display(text)
        v1 = loads(text, mode='strict')[0]
        self.assertEqual(v1.x, 1)
        self.assertEqual(len(v1.y), 7)
        self.assertEqual(all([type(z) is D for z in v1.y]), True)
        self.assertEqual(all([z.z==z1.z for z,z1 in zip(v1.y, lst)]), True)
    #
    def test_5(self):
        from random import randint
        vs = []
        v = C()
        v.x = 1
        v.y = lst = []
        for i in range(7):
            w = D()
            w.z = randint(1,1000)
            lst.append(w)
            vs.append(w)
        vs.append(v)
        text = dumps(vs, crossref=True, pretty=1)
        #print()
        #print(text)
        #vs1 = loads(text, builder='strict')

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(UnsafeLoadsTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

pretty = False

class SetTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_empty_set1(self):
        v = loads('∅')[0]
        self.assertEqual(type(v), set)
        self.assertEqual(v, set())
        s = dumps([v])
        self.assertEqual(s, '∅')
    #
    def test_empty_set2(self):
        v = loads('∅')[0]
        self.assertEqual(type(v), set)
        self.assertEqual(v, set())
        s = dumps([v], pretty=True)
        self.assertEqual(s, '∅')
    #
    def test_set1(self):
        v = loads('{1 2}')[0]
        self.assertEqual(type(v), set)
        self.assertEqual(list(sorted(v)), [1,2])
    #

def suite():
    suite = unittest.SetSuite()
    suite.addTest(unittest.makeSuite(SetTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps

class ConstantTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_nan(self):
        v = loads('NaN')[0]
        s = dumps([v])
        self.assertEqual(s, '?')
    #
    def test_nan2(self):
        v = loads('?')[0]
        s = dumps([v])
        self.assertEqual(s, '?')
    #
    def test_inf(self):
        v = loads('Infinity')[0]
        s = dumps([v])
        self.assertEqual(s, '∞')
    #
    def test_inf1(self):
        v = loads('∞')[0]
        s = dumps([v])
        self.assertEqual(s, '∞')
    #
    def test_ninf(self):
        v = loads('-Infinity')[0]
        s = dumps([v])
        self.assertEqual(s, '-∞')
    #
    def test_ninf1(self):
        v = loads('-∞')[0]
        s = dumps([v])
        self.assertEqual(s, '-∞')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ConstantTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest

import test_int
import test_float
import test_decimal
import test_string
import test_constant
import test_datetime
import test_anonymous
import test_element
import test_mapping
import test_sequence
import test_construct
import test_safe_loads
import test_unsafe_loads
import test_crossref
import test_base64
import test_errors
import test_vs_json
import test_vs_yaml

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_int.IntTestCase))
    suite.addTest(unittest.makeSuite(test_float.FloatTestCase))
    suite.addTest(unittest.makeSuite(test_decimal.DecimalTestCase))
    suite.addTest(unittest.makeSuite(test_string.StringTestCase))
    suite.addTest(unittest.makeSuite(test_constant.ConstantTestCase))
    suite.addTest(unittest.makeSuite(test_datetime.DateTimeTestCase))
    suite.addTest(unittest.makeSuite(test_anonymous.AnonymousTestCase))
    suite.addTest(unittest.makeSuite(test_element.ElementTestCase))
    suite.addTest(unittest.makeSuite(test_mapping.MappingTestCase))
    suite.addTest(unittest.makeSuite(test_sequence.SequenceTestCase))
    suite.addTest(unittest.makeSuite(test_construct.ConstructTestCase))
    suite.addTest(unittest.makeSuite(test_safe_loads.SafeLoadsTestCase))
    suite.addTest(unittest.makeSuite(test_unsafe_loads.UnsafeLoadsTestCase))
    suite.addTest(unittest.makeSuite(test_crossref.CrossrefTestCase))
    suite.addTest(unittest.makeSuite(test_base64.Base64TestCase))
    suite.addTest(unittest.makeSuite(test_errors.LoaderErrorTestCase))
    suite.addTest(unittest.makeSuite(test_vs_json.SimpleONvsJSONTestCase))
    suite.addTest(unittest.makeSuite(test_vs_json.SimpleONvsYAMLTestCase))

    return suite




# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

pretty = False

class ListTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_empty_list1(self):
        v = loads('[]')[0]
        self.assertEqual(type(v), list)
        self.assertEqual(v, [])
        s = dumps([v])
        self.assertEqual(s, '[]')
    #
    def test_empty_list2(self):
        v = loads('[]')[0]
        self.assertEqual(type(v), list)
        self.assertEqual(v, [])
        s = dumps([v], pretty=True)
        self.assertEqual(s, '[]')
    #
    def test_list1(self):
        v = loads('[1 2]')[0]
        self.assertEqual(type(v), list)
        self.assertEqual(v, [1,2])
    #
    def test_list2(self):
        v = loads('[[1 2] [2 3]]')[0]
        self.assertEqual(type(v), list)
        self.assertEqual(v, [[1,2], [2,3]])
    #
    def test_list3(self):
        v = loads('''
            [
                [1 2]
                [2 3]
            ]'''
        )[0]
        self.assertEqual(type(v), list)
        self.assertEqual(v, [[1,2], [2,3]])
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ListTestCase))
    return suite



# coding: utf-8

from __future__ import print_function, unicode_literals

from axon import *

try:
    import builtins
except ImportError:
    import __builtin__ as builtins

try:
    unichr
except:
    unichr = chr

try:
    unicode = builtins.unicode
except AttributeError:
    unicode = builtins.str

import base64
from random import randint

def u(s):
    if type(s) is unicode:
        return s
    else:
        return unicode(s, 'utf-8')

pretty = False
crossref = False

def test_binary():
    print('Test binary')
    text = ''.join([unichr(randint(1,255)) for i in range(256)])
    bstr = text.encode('utf-8')
    print(bstr)
    text1 = '|'+ unicode(base64.encodestring(bstr), 'ascii')
    #print(text, len(bstr), len(text))
    items = loads(text1)
    print(items[0])
    assert items[0] == bstr
    text2 = dumps(items, crossref = crossref)
    assert text1 == text2
    print(text2)

def test(text, pretty=pretty, score=False):
    from time import time

    text = u(text)
    if not score:
        print(">>> ", text)
        item = loads(text)
        #print(item)
        text = dumps(item, pretty=pretty, crossref=crossref)
        #display(text)
        print(text)
        print(type(item[0]))
    else:
        print("Test Json Benchmark string")
        item = loads(text)
        t1 = time()
        N = 100
        for i in range(N):
            loads(text)
        dt1 = time() - t1
        t1 = time()
        for i in range(N):
            dumps(item, pretty=pretty, crossref=crossref)
        dt2 = time() - t1
        print(dt1/N, dt2/N)

def test_random(pretty=pretty):
    from random import choice, uniform
    from time import time

    alpha = [unicode(chr(i)) for i in range(ord('a'),ord('z')+1)]
    def randstr(n, alpha=alpha):
        return ''.join([choice(alpha) for i in range(n)])
    root = []
    m = 20000
    print('Test random items')
    t1 = time()
    for i in range(m):
        seq11 = sequence('bbb', [
            randint(1,1000),
            randint(1,1000),
            randint(1,1000),
        ])
        seq12 = [
            uniform(1.,1000.),
            uniform(1.,1000.),
            uniform(1.,1000.),
            uniform(1.,1000.),
        ]
        ob1 = {'name':randstr(12), 'age':randint(1,100)}
        seq2 = element('test', {'aaa':randstr(16)},
                [randint(1,10000), randint(1,10000),
                 uniform(1.,10000.), uniform(1.,10000.)])
        root.append(
            mapping(
                'alpha',
                {'name': randstr(8), 'id': randstr(4), 'val': seq11, 'val2':seq12,
                'b': ob1, 'others': seq2, 'flag': bool(randint(0,1))})
        )
    dt = time() - t1
    print('Create object', dt)

    t1 = time()
    text = dumps(root, pretty=pretty, crossref = crossref)
    #print(text)
    dt = time() - t1
    N = len(text)
    print('Dump object', dt)
    print(N)

    t1 = time()
    for o in iloads(text):
        pass
    dt = time() - t1
    print('iLoad text', dt, int(N/float(dt)))

    t1 = time()
    root = loads(text)
    dt = time() - t1
    print('Load text', dt, int(N/float(dt)))

    t1 = time()
    text = dumps(root, pretty=pretty, crossref = crossref)
    dt = time() - t1
    N = len(text)
    print('Dump object again', dt)


def test2():
    #from random import randint
    from time import time

    m = 40000
    root = []
    t1 = time()
    for i in range(m):
        seq1 = sequence('aaa', [123, 456, 45.1256421])
        ob1 = {'name':'belinda', 'age':65}
        seq2 = element('test', {'aaa':'qwewqewqeqw'}, [345, 789])
        root.append(
            mapping(
                'alpha',
                {'name': 'john', 'id': 'a', 'val': seq1,
                'b': ob1, 'others': seq2, 'flag': True})
        )
    dt = time() - t1
    print('Create object', dt)

    t1 = time()
    text = dumps(root, pretty=pretty, crossref = crossref)
    #print(text)
    dt = time() - t1
    N = len(text)
    print('Dump object', dt)
    print(N)

    t1 = time()
    v = [0]
    for o in iloads(text):
        pass
    dt = time() - t1
    print('iLoad text', dt, int(N/float(dt)))

    t1 = time()
    root=loads(text)
    dt = time() - t1
    print('Load text', dt, int(N/float(dt)))

    t1 = time()
    text = dumps(root, pretty=pretty, crossref = crossref)
    dt = time() - t1
    N = len(text)
    print('Dump object again', dt)

task_config = '''
config {
    data {
        type:"natural_fire"
        kind:"monthly"
        period:[5 6 7 8]
        years:[2001 2002 2003 2004 2005 2006 2007 2008]
        attrs:[temp osad]
        attr:count
        attr2:{a:1 b:1}
        [1 2 3]
        {a:1 b:2}
    }
    graph {
        [1 2 3]
        {a:1 b:2}
        type:"scatter"
        schema:"xy-z"
    }
    curve {rbf:{kind: "LS" gamma: 1.0}}
}
'''

json_string = """
{
    web_app: [
        servlet {
            name:"cofaxCDS"
            class:'org.cofax.cds.CDSServlet'
            init_param {
                configGlossary : {
                    installationAt:"Philadelphia PA"
                    adminEmail:"ksm@pobox.com"
                    poweredBy:"Cofax"
                    poweredByIcon:"/images/cofax.gif"
                    staticPath:"/content/static"}
                templateProcessorClass:"org.cofax.WysiwygTemplate"
                templateLoaderClass:"org.cofax.FilesTemplateLoader"
                templatePath:"templates"
                templateOverridePath:""
                defaultListTemplate:"listTemplate.htm"
                defaultFileTemplate:"articleTemplate.htm"
                useJSP:false
                jspListTemplate:"listTemplate.jsp"
                jspFileTemplate:"articleTemplate.jsp"
                cachePackageTagsTrack:200
                cachePackageTagsStore:200
                cachePackageTagsRefresh:60
                cacheTemplatesTrack:100
                cacheTemplatesStore:50
                cacheTemplatesRefresh:15
                cachePagesTrack:200
                cachePagesStore:100
                cachePagesRefresh:10
                cachePagesDirtyRead:10
                searchEngineListTemplate:"forSearchEnginesList.htm"
                searchEngineFileTemplate:"forSearchEngines.htm"
                dataStoreDriver:'com.microsoft.jdbc.sqlserver.SQLServerDriver'
                dataStoreUrl:'jdbc:microsoft:sqlserver://LOCALHOST:1433;DatabaseName:goon'
                dataStoreUser:"sa"
                dataStorePassword:"dataStoreTestQuery"
                dataStoreTestQuery:"SET NOCOUNT ON;select test='test';"
                dataStoreLogFile:"/usr/local/tomcat/logs/datastore.log"
                dataStoreInitConns:10
                dataStoreMaxConns:100
                dataStoreConnUsageLimit:100
                dataStoreLogLevel:"debug"
                maxUrlLength:500
            }
            servlet {
                name:"cofaxEmail"
                class:"org.cofax.cds.EmailServlet"
                init_param {
                    mailHost:"mail1"
                    mailHostOverride:"mail2"
                }
            }
            servlet{
                name:"cofaxAdmin"
                class:"org.cofax.cds.AdminServlet"
            }
            servlet{
                name:"fileServlet"
                class:"org.cofax.cds.FileServlet"
            }
            servlet{
                name:"cofaxTools"
                class:"org.cofax.cms.CofaxToolsServlet"
                init_param {
                    templatePath:"toolstemplates/"
                    log:1
                    logLocation:"/usr/local/tomcat/logs/CofaxTools.log"
                    logMaxSize:""
                    dataLog:1
                    dataLogLocation:"/usr/local/tomcat/logs/dataLog.log"
                    dataLogMaxSize:""
                    removePageCache:"/content/admin/remove?cache:pages&id:"
                    removeTemplateCache:"/content/admin/remove?cache:templates&id:"
                    fileTransferFolder:"/usr/local/tomcat/webapps/content/fileTransferFolder"
                    lookInContext:1
                    adminGroupID:4
                    betaServer:true
                }
            }
        }
        servlet_mapping {
            cofaxCDS:"/"
            cofaxEmail:"/cofaxutil/aemail/*"
            cofaxAdmin:"/admin/*"
            fileServlet:"/static/*"
            cofaxTools:"/tools/*"
        }
        taglib {
            taglib_uri:"cofax.tld"
            taglib_location:"/WEB-INF/tlds/cofax.tld"
        }
    ]
}
"""

def main():
    # samples
    test('0')
    test('-0')
    test('17')
    test('171717171717171717171718181818181818')
    test('-17')
    test('-17.34')
    test('17.34')
    test('-1e-6')
    test('1.734E12')
    test('-1.734e-12')
    test('-17.34$')
    test('-1.0e-6$')
    test('1.734E12$')
    test('1e-12$')
    test("NaN")
    test("Infinity")
    test("-Infinity")
    test('null')
    test('{}')
    test('"abc"')
    test('"a\tbc"')
    test("2010-10-01")
    test("10:10")
    test("10:10:20")
    test("10:10:20.100")
    test("10:10:20.3000")
    test("2010-10-01T10:10")
    test("2010-10-01T10:10:20")
    test(
        '"asasasas1\n # ddddd\\"dddddddd2\n @ sasafgfghgfhg3\n ! xvxcvcxvcv4"'
    )
    test(
        '"asasasas1\n # ddddd\\"dddddddd2\n @ sasafgfghgfhg3\n ! xvxcvcx\\"\\"vcv4"'
    )
    test('"\\u0041\\U0042"')
    test('true')
    test('false')
    test('[1 2 3]')
    test('[ 1 2 3 ]')
    test(
    '''[
        1
        2
        3]''')
    #test("'nn'{1 2 3 *}")
    test('nn {1 2 3}')
    test("'name example123'{}")
    test('{ a:22 b:32}')
    test('{a:22 b:32}')
    test('''aaa{a:22
    b:32}''')
    test("{a:[11 22 33]}")
    test('hel_lo{}')
    test('hello_world{ a:22 b:"http://www.python.org"}')
#    test('''{
# a:102
#
#ppp:
#{ abb:11 ccc:a}
#}
#''')
    test(
'''{
width:100
height:200
image: { type:"svg"}
}''')
    test('foo{ a:13 b:23 c:[1 2 3]}')

    test('_{1 2 3 a:1 b:"OPT"}')
    test('nn {1 2 3 a:1 b:"OPT"}')
    test('''
    [
        [1 2 3 -7 2.5 3.0]
        [5 6 7 -7 2.5 3.0]
    ]''')
    test('''
    dataset {
        name:"archive_pogoda"
        fields: [id date time T R24 W]
        types: [int date time float float ["W" "E"]]

        [1232 2009-01-12 12:00 -7 2.5 "W"]
        [1234 2009-01-12 18:00 -4 3.5 "E"]
    }
    ''')
    test('{win: ["Windows NT" "Windows XP"] linux: ["ALT Linux" "RedHat"] mac:"OS X"}')
    test('''ABC {
        CDE { b:4 c:4}
        CDE { b:3 c:6}
    }''')
    test('''
    type {
        name:"mytype"
        attribute { name:"x" type:"int" required:true }
        element { name:"folder" multiple:"true" }
    }''')
    test('''
    html{
        h1{"Page title"}
        table{
            width:100
            tr{
                td{"cell 1 1"}
                td{"cell 1 2" "cell 1 2"}
                td{"cell 1 3"}
            }
            tr{
                td{"cell 2 1"}
                td{colspan:2 "cell 2 2"}
            }
        }
    }''')
#     test('''[
#     S{ &id123 1 2 3}
#     U{ *id123 }
#     ]
#     ''')
#     test('''[
#     S{ &id123 1 2 3}
#     U{ aaa: *id123 *id123 }
#     ]
#     ''')
    test('''[tr{} tr{} tr{}]''')
    test('''_{name:'nemo' tr{} tr{} tr{}}''')
    test('''aaa{name:'nemo' age:32 tr{} tr{} tr{}}''')
    test('aaa{}')
    test('aaa{a:1}')
    test(task_config)
    test(json_string)
    test_binary()

    test2()
    test_random()

if __name__ == '__main__':
    #test2()
    main()

#figure {size:[10.0 9.0] dpi:100
#    lines {
#        line {1.2 2.3 3.4 4.5 style:"" color:""}
#    }
#    polygon {background:"r" foreground:"b"
#        [1.1 2.2] [3.2 1.4] [3.2 4.5]
#    }
#}




# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

class ElementTestCase(unittest.TestCase):

    def setUp(self):
        pass
    #
    def test_empty_element(self):
        v = element('aaa', {}, [])
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(v.mapping, {})
        self.assertEqual(v.sequence, [])
    #
    def test_element_init(self):
        l = {'a':1, 'b':2}
        v = element('aaa', {'a':1, 'b':2}, [1,2,3])
        self.assertEqual(v.mapping, {'a':1, 'b':2})
        self.assertEqual(v.sequence, [1,2,3])
    #
    def test_element1(self):
        v = loads('aaa{a:1 b:2 c:3 4 5 6}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Element)
        self.assertEqual(v.sequence, [4, 5, 6])
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v])
        self.assertEqual(s, "aaa{a:1 b:2 c:3 4 5 6}")
    #
    def test_element1_1(self):
        v = loads('''
aaa:
   a:1
   b:2
   c:3
   4 5 6
''')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Element)
        self.assertEqual(v.sequence, [4, 5, 6])
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v])
        self.assertEqual(s, \
            'aaa{' + \
            ' '.join("%s:%s" % (k,x) for k,x in sorted(v.mapping.items())) + ' ' + \
            ' '.join("%s" % x for x in v.sequence) + \
            '}')
    #
    def test_element2(self):
        v = loads('aaa {a:1 b:2 c:3 4 5 6}')[0]
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(type(v), Element)
        self.assertEqual(v.sequence, [4, 5, 6])
        self.assertEqual(v.mapping, {'a': 1, 'b': 2, 'c': 3})
        s = dumps([v], pretty=2)
        self.assertEqual(s, \
            'aaa:\n  ' + \
            '\n  '.join(\
                      "%s: %s" % (k,x) \
                      for k,x in sorted(v.mapping.items())) + \
            '\n  ' + \
            '\n  '.join("%s" % x for x in v.sequence)
            )
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ElementTestCase))
    return suite



# coding: utf-8

try:
    import builtins
except ImportError:
    import __builtin__ as builtins

import datetime

try:
    from cdecimal import Decimal
except:
    from decimal import Decimal

try:
    unicode_type = builtins.unicode
    str_type = builtins.str
except AttributeError:
    unicode_type = builtins.str
    str_type = builtins.str

try:
    long_type = builtins.long
    int_type = builtins.int
except AttributeError:
    int_type = builtins.int
    long_type = builtins.int

float_type = builtins.float
bool_type = builtins.bool
bytes_type = type('a'.encode('ascii'))
date_type = datetime.date
time_type = datetime.time
datetime_type = datetime.datetime
decimal_type = Decimal



# coding: utf-8

import pstats, cProfile

import test_vs_json as test

cProfile.runctx("test.run()", globals(), locals(), "Profile.prof")

s = pstats.Stats("Profile.prof")
s.strip_dirs().sort_stats("time").print_stats()



# coding: utf-8

from distutils.core import setup
from distutils.extension import Extension

#from Cython.Build import cythonize
#from Cython.Compiler import Options
#Options.fast_fail = True
#Options.binding = False

import os

ext_modules = [
    Extension(
        "axon._objects",
        ["lib/axon/_objects.c"]
    ),
    Extension(
        "axon._loader",
        ["lib/axon/_loader.c"]
    ),
    Extension(
        "axon._dumper",
        ["lib/axon/_dumper.c"]
    ),
]

#ext_modules = cythonize(['lib/_objects.py', 'lib/_loader.py', 'lib/_dumper.py'])

long_description = '''\
Python library for `AXON <http://axon.intellimath.org>`_.

An eXtended Object Notation (``AXON``) is simple text based format for interchanging
objects, documents and data.
'''

setup(
    name = 'pyaxon',
    version = '0.5',
    description = 'Python library for An eXtended Object Notation (AXON)',
    author = 'Zaur Shibzukhov',
    author_email="szport@gmail.com",
    maintainer="Zaur Shibzukhov",
    maintainer_email="szport@gmail.com",
    license="MIT License",
    ext_modules = ext_modules,
    package_dir = {'': 'lib'},
    packages = ['axon', 'axon.test', 'axon.test.benchmark'],
    url = 'http://axon.intellimath.org',
    long_description = long_description,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



# coding: utf-8

from __future__ import unicode_literals

import sys

class LoaderError(Exception):
    #
    def __init__(self, lnum, pos, msg, chunk):
        self.msg = msg
        self.lnum = lnum
        self.pos = pos
        self.chunk = chunk
    #
    def __str__(self):
        return 'ERROR:: %s. line:%s pos:%s chunk: %s\n' % \
                    (self.msg, self.lnum, self.pos+1, repr(self.chunk))

def error(self, msg):
    e = LoaderError(self.lnum, self.pos, msg, self.line[self.pos:self.pos+16])
    self.errto.write(str(e))
    raise e
#
# def error_getvalue(self):
#     error(self, 'Invalid simple value')
#
def error_unexpected_end(self):
    error(self, 'Unexpected end')
#
def error_getnumber(self):
    error(self, 'Invalid number')
#
def error_getint_part(self):
    error(self, 'Invalid int part or value')
#
def error_invalid_date(self):
    error(self, 'Invalid date')
#
def error_invalid_time(self):
    error(self, 'Invalid time')
#
def error_invalid_datetime(self):
    error(self, 'Invalid datetime')
#
def error_invalid_value(self, vtype=''):
    error(self, 'Invalid %s value' % vtype)
#
def error_invalid_value_with_prefix(self, prefix):
    error(self, "Invalid value with prefix '%s'" % prefix)
#
def error_invalid_value_star(self):
    error(self, 'Invalid value before *')
#
def error_dict_value(self):
    error(self, 'dict can contain only key:value pairs')
#
def error_unexpected_attribute(self, name):
    error(self, 'Unexpected attribute %s:?' % name)
#
def error_unexpected_value(self, context=''):
    error(self, 'Unexpected value: %s' % context)
#
def error_end_item(self):
    error(self, "Expected space character, '[', '}' or ')' here")
#
def error_star(self):
    error(self, 'Invalid usage of symbol *')
#
def error_undefined_name(self, name):
    error(self, 'Undefined name %r' % name )
#
def error_indentation(self, idn):
    error(self, 'Invalid indentation: current position=%s ident position=%s' % (self.pos, idn))
#
def error_expected_name(self):
    error(self, 'Expected name here')
#
def error_expected_same_name(self, name):
    error(self, "Element must have same name as '%s'" % name)
#
def error_expected_complex_value(self):
    error(self, "Expected complex here")



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

pretty = False

class SafeLoadsTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_empty1(self):
        v = loads('aaa{}')[0]
        self.assertEqual(type(v), Empty)
        self.assertEqual(v.name, 'aaa')
        self.assertEqual(v.sequence, [])
        self.assertEqual(v.mapping, {})
        s = dumps([v])
        self.assertEqual(s, 'aaa{}')
    #
    def test_empty2(self):
        v = loads('aaa{}')[0]
        self.assertEqual(type(v), Empty)
        self.assertEqual(v.sequence, [])
        self.assertEqual(v.mapping, {})
        s = dumps([v], pretty=1)
        self.assertEqual(s, 'aaa{}')
    #
    def test_empty3(self):
        v = loads('''
aaa:
   ...
''')[0]
        self.assertEqual(type(v), Empty)
        self.assertEqual(v.sequence, [])
        self.assertEqual(v.mapping, {})
        s = dumps([v], pretty=1)
        self.assertEqual(s, 'aaa{}')
    #
    def test_empty4(self):
        v = loads('''
aaa:
  ...
''')[0]
        self.assertEqual(type(v), Empty)
        self.assertEqual(v.sequence, [])
        self.assertEqual(v.mapping, {})
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  ...''')
    #
    def test_empty5(self):
        v = loads('''
aaa:
  bbb:
    ...
  ccc:
    ...
''')[0]
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  bbb:
    ...
  ccc:
    ...''')
    #
    def test_empty6(self):
        v = loads('''
aaa:
  a: 1
  b:2
  bbb:
    ...
  ccc:
    c:1
    ddd:
      ...
''')[0]
        s = dumps([v], pretty=2)
        self.assertEqual(s, '''\
aaa:
  a: 1
  b: 2
  bbb:
    ...
  ccc:
    c: 1
    ddd:
      ...''')
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SafeLoadsTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps
from datetime import date, time, datetime, tzinfo

class DateTimeTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_date1(self):
        v = loads('2010-12-01')[0]
        self.assertEqual(type(v), date)
        s = dumps([v])
        self.assertEqual(s, '2010-12-01')
    #
    def test_date2(self):
        v = loads('1900-01-01')[0]
        self.assertEqual(type(v), date)
        s = dumps([v])
        self.assertEqual(s, '1900-01-01')
    #
    def test_date3(self):
        v = loads('12-01-01')[0]
        self.assertEqual(type(v), date)
        s = dumps([v])
        self.assertEqual(s, '12-01-01')
    #
    def test_date4(self):
        v = loads('0-00-00')[0]
        self.assertEqual(type(v), date)
        s = dumps([v])
        self.assertEqual(s, '0-00-00')
    #
    def test_time1(self):
        v = loads('00:00')[0]
        self.assertEqual(type(v), time)
        s = dumps([v])
        self.assertEqual(s, '00:00')
    #
    def test_time2(self):
        v = loads('23:59:59')[0]
        self.assertEqual(type(v), time)
        s = dumps([v])
        self.assertEqual(s, '23:59:59')
    #
    def test_time3(self):
        v = loads('23:59:59.000123')[0]
        self.assertEqual(type(v), time)
        s = dumps([v])
        self.assertEqual(s, '23:59:59.000123')
    #
    def test_time4(self):
        v = loads('23:59:59+00:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59+00')
        self.assertEqual(v.utcoffset().seconds, 0)
    #
    def test_time5(self):
        v = loads('23:59:59+01:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59+01')
        self.assertEqual(v.utcoffset().seconds/60, 60)
    #
    def test_time6(self):
        v = loads('23:59:59-01:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59-01')
        self.assertEqual(v.utcoffset().seconds/60, 23*60)
    #
    def test_time7(self):
        v = loads('23:59:59+12:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59+12')
        self.assertEqual(v.utcoffset().seconds/60, 12*60)
    #
    def test_time8(self):
        v = loads('23:59:59+23:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59-01')
        self.assertEqual(v.utcoffset().seconds/60, 23*60)
    #
    def test_time9(self):
        v = loads('23:59:59-23:00')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59+01')
        self.assertEqual(v.utcoffset().seconds/60, 60)
    #
    def test_time10(self):
        v = loads('23:59:59+3:15')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59+03:15')
        self.assertEqual(v.utcoffset().seconds/60, 3*60+15)
    #
    def test_time11(self):
        v = loads('23:59:59-3:15')[0]
        self.assertEqual(type(v), time)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '23:59:59-03:15')
        self.assertEqual(v.utcoffset().seconds/60, 1440-3*60-15)
    #
    def test_datetime1(self):
        v = loads('2010-01-01T00:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertEqual(v.tzinfo, None)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T00:00')
    #
    def test_datetime2(self):
        v = loads('1-01-01T23:59:59')[0]
        self.assertEqual(type(v), datetime)
        self.assertEqual(v.tzinfo, None)
        s = dumps([v])
        self.assertEqual(s, '1-01-01T23:59:59')
    #
    def test_datetime3(self):
        v = loads('2010-01-01T23:59:59.000123')[0]
        self.assertEqual(type(v), datetime)
        self.assertEqual(v.tzinfo, None)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59.000123')
    #
    def test_datetime4(self):
        v = loads('2010-01-01T23:59:59+00:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59+00')
        self.assertEqual(v.utcoffset().seconds, 0)
    #
    def test_datetime5(self):
        v = loads('2010-01-01T23:59:59+01:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59+01')
        self.assertEqual(v.utcoffset().seconds/60, 60)
    #
    def test_datetime6(self):
        v = loads('2010-01-01T23:59:59-01:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59-01')
        self.assertEqual(v.utcoffset().seconds/60, 23*60)
    #
    def test_datetime7(self):
        v = loads('2010-01-01T23:59:59+12:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59+12')
        self.assertEqual(v.utcoffset().seconds/60, 12*60)
    #
    def test_datetime8(self):
        v = loads('2010-01-01T23:59:59+23:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59-01')
        self.assertEqual(v.utcoffset().seconds/60, 23*60)
    #
    def test_datetime9(self):
        v = loads('2010-01-01T23:59:59-23:00')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59+01')
        self.assertEqual(v.utcoffset().seconds/60, 60)
    #
    def test_datetime10(self):
        v = loads('2010-01-01T23:59:59+3:15')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59+03:15')
        self.assertEqual(v.utcoffset().seconds/60, 3*60+15)
    #
    def test_datetime11(self):
        v = loads('2010-01-01T23:59:59-3:15')[0]
        self.assertEqual(type(v), datetime)
        self.assertIsInstance(v.tzinfo, tzinfo)
        s = dumps([v])
        self.assertEqual(s, '2010-01-01T23:59:59-03:15')
        self.assertEqual(v.utcoffset().seconds/60, 1440-3*60-15)
    #

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DateTimeTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import *

pretty = False

class CrossrefTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_crossref_1(self):
        v1, v2 = loads('&123 [ 17] a{ *123 "abc" }')
        self.assertEqual(v1, v2.sequence[0])
    #
    def test_crossref_2(self):
        v1, v2 = loads('&_123_ a{ a:1 } b{ *_123_ "abc" }')
        self.assertEqual(v1, v2.sequence[0])
    #
    def test_crossref_3(self):
        v1, v2 = loads('&123 a{ a:1 100 200} b{ *123 "abc" }')
        self.assertEqual(v1, v2.sequence[0])


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CrossrefTestCase))
    return suite



# coding: utf-8

from __future__ import unicode_literals
import unittest
from axon import loads, dumps
from axon.errors import LoaderError

class LoaderErrorTestCase(unittest.TestCase):

    def setUp(self):
        pass

    def test_unexpected_end_1(self):
        text = 'a{'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_11(self):
        text = '{'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_12(self):
        text = '['
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_2(self):
        text = '"abc'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_3(self):
        text = '''"abc
        qwertyuiop
        '''
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_4(self):
        text = 'a{name:'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_41(self):
        text = '{name:'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_5(self):
        text = 'a{name:}'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_51(self):
        text = '{name:}'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_52(self):
        text = '{name:"anne" 17}'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_6(self):
        text = 'a{name:1 age:'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_61(self):
        text = '{name:1 age:'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_7(self):
        text = '[1 2 '
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_71(self):
        text = '[1 2 a:'
        try:
            vs = loads(text)
        except LoaderError:
            pass

    def test_unexpected_end_72(self):
        text = '[1 2 a:7'
        try:
            vs = loads(text)
        except LoaderError:
            pass

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LoaderErrorTestCase))
    return suite
